@@ -3,7 +3,7 @@ controls.py - support classes for LDAPv3 extended operations
 
 See http://www.python-ldap.org/ for details.
 
-\$Id: __init__.py,v 1.3 2011/04/02 22:45:53 stroeder Exp $
+\$Id: __init__.py,v 1.4 2011/07/22 13:27:02 stroeder Exp $
 
 Description:
 The ldap.extop module provides base classes for LDAPv3 extended operations.
@@ -16,7 +16,13 @@ from ldap import __version__
 
 class ExtendedRequest:
   """
-  Generic base class for a LDAP extended operation request
+  Generic base class for a LDAPv3 extended operation request
+
+  requestName
+      OID as string of the LDAPv3 extended operation request
+  requestValue
+      value of the LDAPv3 extended operation request
+      (here it is the BER-encoded ASN.1 request value)
   """
 
   def __init__(self,requestName,requestValue):
@@ -27,12 +33,21 @@ class ExtendedRequest:
     return '%s(%s,%s)' % (self.__class__.__name__,self.requestName,self.requestValue)
 
   def encodedRequestValue(self):
+    """
+    returns the BER-encoded ASN.1 request value composed by class attributes
+    set before
+    """
     return self.requestValue
 
 
 class ExtendedResponse:
   """
-  Generic base class for a LDAP extended operation response
+  Generic base class for a LDAPv3 extended operation response
+
+  requestName
+      OID as string of the LDAPv3 extended operation response
+  encodedResponseValue
+      BER-encoded ASN.1 value of the LDAPv3 extended operation response
   """
 
   def __init__(self,responseName,encodedResponseValue):
@@ -43,6 +58,10 @@ class ExtendedResponse:
     return '%s(%s,%s)' % (self.__class__.__name__,self.responseName,self.responseValue)
 
   def decodeResponseValue(self,value):
+    """
+    decodes the BER-encoded ASN.1 extended operation response value and
+    sets the appropriate class attributes
+    """
     return value
 
 
