@@ -1,5 +1,5 @@
 /* See http://www.python-ldap.org/ for details.
- * $Id: ldapcontrol.h,v 1.5 2009/04/17 12:19:09 stroeder Exp $ */
+ * $Id: ldapcontrol.h,v 1.6 2009/08/04 05:39:10 leonard Exp $ */
 
 #ifndef __h_ldapcontrol
 #define __h_ldapcontrol
@@ -9,7 +9,7 @@
 
 void LDAPinit_control(PyObject *d);
 void LDAPControl_List_DEL( LDAPControl** );
-LDAPControl** List_to_LDAPControls( PyObject* );
+int  LDAPControls_from_object(PyObject *, LDAPControl ***);
 PyObject* LDAPControls_to_List(LDAPControl **ldcs);
 
 #endif /* __h_ldapcontrol */
