@@ -1,5 +1,5 @@
 /* See http://www.python-ldap.org/ for details.
- * $Id: message.h,v 1.5 2009/04/17 12:19:09 stroeder Exp $ */
+ * $Id: message.h,v 1.6 2011/02/21 21:04:00 stroeder Exp $ */
 
 #ifndef __h_message 
 #define __h_message 
@@ -8,7 +8,7 @@
 #include "lber.h"
 #include "ldap.h"
 
-extern PyObject* LDAPmessage_to_python( LDAP*ld, LDAPMessage*m );
+extern PyObject* LDAPmessage_to_python( LDAP*ld, LDAPMessage*m, int add_ctrls, int add_intermediates );
 
 #endif /* __h_message_ */
 
